      subroutine hirprw7_Hv2(iu,nu,linmax,lonfix,lcoeff,noflip)
c ... preprocess hirs data ... apply limb (optional), reflected sunlight,
c ... and fluorescence (optional) corrections.
c ... version of 19.05.94
c ... parameters ..
c
c     This version includes
c          1.  a fatal error test not included in ITPP package
c          2.  Added iobuf2 parameter to eliminate bug in output data.
c          3.  Removed all msu components to this subroutine.
c          4.  Arguement list does not contain 1 MSU parameter.
c          5.  writes time data in position 24
c          6.  writes altitude in position 25
c          7.  writes scan position + line number in position 26
c          8.  Removes test that removes all TBs if one channel missing
c              (irok flag)
c
      integer leno,lens,np,nid,nnd
      parameter (leno=112,lens=leno/4,np=30,nid=3,nnd=6)
      integer nc,nl,ns,nm
      parameter (nc=19,nl=2,ns=56,nm=7)
      integer maxlin,maxsec
      parameter (maxlin=1100,maxsec=maxlin/2)
c ...
c ... scalar arguments ..
      integer iu,lcoeff,linmax,lonfix,noflip,nu
c ...
c ... scalars in common ..
      integer lchoff,lcmoff
c ...
c ... arrays in common ..
      integer locmap(ns,nl,maxsec)
c ...
c ... local scalars ..
      real cosz,ss,sind,cosd,g
      integer i,ibdr,idatb,idir,iflag,ii,ilat,ilon,incs,inihlc,irbad,
     +        irec,is,isat,isec,isp,iszn,itc4,itimb,itime,izen,j,jj,jl,
     +        jsub,k,kfluor,ksec,lflag,lin,lines,linex,loc,locl,lsec,lx,
     +        misg,msec,msect,msghir,mtime,n,nite,nsec,nsect,ialt
      logical hirsok,irok,locok
c ...
c ... local arrays ..
      integer idt(nid),ifill(np),iobuf(leno),isbuf(lens),item(np),
     +        jbuf(ns,nl,np),locbuf(ns,nl),ndt(nnd)
      integer iobuf2(leno)
c ...
c ... external functions ..
      integer ibiref,ifluor,isolzn_pc,lbit
      external ibiref,ifluor,isolzn_pc,lbit
c ...
c ... external subroutines ..
      external dattim,hirlc3,pfcoef_pc
c ...
c ... intrinsic functions ..
      intrinsic float,iabs,mod
c ...
c ... common blocks ..
      common /locdir/locmap
      common /nohlc/lchoff
      common /nomlc/lcmoff
      common /iso/ss,sind,cosd,g

c ...
c ... data statements ..
      data nite/9000/,ifill/np*999999/,misg/999999/
      data inihlc/0/
c ...
 
c      write (*,fmt='('' begin hirs preprocessing'')')
c ... read header record
      irec = 1
      read (iu,rec=irec) iobuf
c ... extract needed information
      isat = iobuf(1)
      idir = iobuf(3)
      if (idir.ne.1) idir = 2
      nsect = iobuf(2)
      msect = linmax/2
c ... set up initial record + record-increment for output ...
c ... data are arranged in output file with correct orientation for
c ... video -- i.e., ascending orbit is flipped top-to-bottom,
c ... descending orbit is flipped left-to-right, unless noflip.ne.0
      if (noflip.ne.0) go to 120
      go to (110,120) idir
  110 msec = msect
      incs = -1
      go to 130
  120 msec = 1
      incs = 1
  130 msec = msec + 1
c ... complete initialization
      call pfcoef_pc(isat)
      msghir = 0
      locl = 0
      lines = 0
      linex = 1
      lsec = -1
c ... main processing loop
      do 370 lx = linex,linmax,2
        locl = locl + 1
        do 150 j = 1,nl
          do 150 i = 1,ns
            do 140 k = 1,np
  140       jbuf(i,j,k) = ifill(k)
  150   locbuf(i,j) = locmap(i,j,locl)
c ...   build a two-line buffer for output
        do 270 lin = 1,nl
          do 260 isp = 1,ns
            loc = locbuf(isp,lin)
            if (loc.ne.0) go to 170
  160       msghir = msghir + 1
            go to 260
  170       jsub = loc/65536
            isec = mod(loc,65536)
            if (isec.eq.lsec) go to 180
            lsec = isec
            read (iu,rec=isec) iobuf
  180       jj = lens* (jsub-1)
            do 190 i = 1,lens
  190       isbuf(i) = iobuf(i+jj)
c ...       extract parameters
            lline=lx-1+lin
            irbad = 0
            do 200 i = 1,nc
              item(i) = isbuf(i+1)
c              if (item(i).eq.0 .or. item(i).eq.misg) irbad=irbad+1
              if (item(i).eq.0) item(i)=misg 
  200       continue
            irok = irbad .eq. 0
            item(20) = isbuf(21)
            ilat = isbuf(22)
            ilon = isbuf(23)
            izen = iabs(isbuf(24))
            iflag = isbuf(25)
            ialt = isbuf(28)
            locok = (iabs(ilat).le.9000) .and. (iabs(ilon).le.18000)
            if (.not.locok) go to 160
c ...       idt(1)=seconds of day, idt(2)=day of year, idt(3)=year
            idt2 = isbuf(27)               ! Preserve time in millisecond
            idt(1) = float(isbuf(27))/64.
            idt(2) = mod(isbuf(26),1000)
            idt(3) = isbuf(26)/1000
c ...       check ingest quality flag for other 'fatal' conditions
            hirsok = .true.
            if (iflag.eq.0) go to 210
            lflag = 0
            lflag = lflag + lbit(25,iflag)
            lflag = lflag + lbit(27,iflag)
            lflag = lflag + lbit(31,iflag)  ! fatal err test added 7/14/98
            lflag = lflag + lbit(32,iflag)
            hirsok = lflag .eq. 0
  210       continue
c ...       process data and store in staging buffer
            if(hirsok) go to 230
c            if (hirsok .and. irok) go to 230
            do 220 i = 1,nc
  220       item(i) = ifill(i+3)
  230       continue
            iszn = ifill(3)
            ibdr = ifill(23)
            itc4 = ifill(24)
c ...       compute solar zenith angle ... if daytime, correct 2360
c ...       channel (tirosn - noaa10) for fluorescence (before limb-correction)
c
c ...       compute bidirectional reflectance from visible channel
c ...       radiance, and correct 2511 channel for reflected sunlight
c ...       after limb-correction).
            iszn = isolzn_pc(ilat,ilon,idt,cosz)
            if (iszn.eq.nite) go to 240
            if (.not.hirsok) go to 240
            call dattim(idt,ndt)
            ibdr = ibiref(item(20),ndt(2),cosz)
c            if (.not.irok) go to 240
c ...       LG 31-mar-95 note the next statement was added to ensure
c ...                    consistent behaviour - kfluor was not set
c ...                    previously
            kfluor = 0
c ...            
            if (kfluor.eq.0) go to 240
            if (isat.lt.11 .or. isat.eq.12) item(17) = ifluor(item(17),
     +          cosz)
  240       continue
            jbuf(isp,lin,1) = ilat
            jbuf(isp,lin,2) = ilon
            jbuf(isp,lin,3) = iszn
            do 250 k = 1,nc
  250       jbuf(isp,lin,k+3) = item(k)
            jbuf(isp,lin,23) = ibdr
c            jbuf(isp,lin,24) = itc4 
            jbuf(isp,lin,24) = idt2      ! Time in seconds * 64
            jbuf(isp,lin,25) = ialt      ! altitude in meters
            jbuf(isp,lin,26) = (lline*100)+isp  ! scan position + line
  260     continue
  270   continue
c......Applies HIRS limb correction.
        call hirlc3(jbuf,isat,inihlc)
c ...   finished two lines ... ready for output
        ksec = msec
        if (noflip.ne.0) go to 320
        go to (280,320) idir
c ...   ascending ... exchange lines within pair
  280   do 310 k = 1,np
          jj = 0
          do 300 jl = 1,nl
            j = 3 - jl
            do 290 is = 1,ns
              ii = is + jj
              i = is
  290       iobuf2(ii) = jbuf(i,j,k)
  300     jj = jj + ns
          write (nu,rec=ksec) iobuf2
  310   ksec = ksec + maxsec
        go to 360
c ...   descending ... reverse order of spots within line
c ...   or
c ...   no flip ... retain original order
  320   do 350 k = 1,np
          jj = 0
          do 340 jl = 1,nl
            j = jl
            do 330 is = 1,ns
              ii = is + jj
              if (noflip.eq.0) then
                i = ns + 1 - is
              else
                i = is
              end if
  330       iobuf2(ii) = jbuf(i,j,k)
  340     jj = jj + ns
          write (nu,rec=ksec) iobuf2
  350   ksec = ksec + maxsec
c ...   adjust master sector index for next round
  360   msec = msec + incs
        lines = lines + nl
c        if (mod(lines,100).eq.0) write (*,fmt=375) lines
  370 continue
  375 format (' no.lines completed = ',i4)
      if (linmax.eq.maxlin) go to 410
c ... fill unused portion of output file
      do 380 k = 1,np
        do 380 j = 1,nl
          do 380 i = 1,ns
  380 jbuf(i,j,k) = ifill(k)
      ksec = msect + nl
      nsec = maxsec - ksec
      do 400 k = 1,np
        lsec = ksec + nsec + 1
        do 390 n = ksec,lsec
          write (nu,rec=n) ((jbuf(i,j,k),i=1,ns),j=1,nl)
  390   continue
  400 ksec = ksec + maxsec
c ... finished processing ... construct & output header record
  410 irec = 1
      read (iu,rec=irec) iobuf
      idt(1) = float(iobuf(4))/64.
      idt(2) = mod(iobuf(5),1000)
      idt(3) = iobuf(5)/1000
      call dattim(idt,ndt)
c ... idatb=ndt(1)*10000+ndt(2)*100+ndt(3)
      idatb = iobuf(5)
      itimb = ndt(4)*10000 + ndt(5)*100 + ndt(6)
      idt(1) = float(iobuf(6))/64.
      idt(2) = mod(iobuf(7),1000)
      idt(3) = iobuf(7)/1000
      call dattim(idt,ndt)
      itime = ndt(4)*10000 + ndt(5)*100 + ndt(6)
      mtime = iobuf(13)
      do 420 i = 1,leno
  420 iobuf(i) = 0
      iobuf(1) = idatb
      iobuf(2) = itimb
      iobuf(3) = itime
      iobuf(4) = lines
      iobuf(5) = isat
c ... set status to 'ready-to-process'
      iobuf(6) = 1
      iobuf(7) = idir
c ... start-time in milliseconds
      iobuf(50) = mtime
c ... store limb-correction flags
      iobuf(51) = lchoff
      iobuf(52) = lcmoff
c ... store image-rectification flag
      iobuf(53) = noflip
c ... store coefficient-type flag
      iobuf(54) = lcoeff
      write (nu,rec=irec) iobuf
c      write (*,fmt=450) msghir
c      write (*,fmt=470) (iobuf(i),i=1,nm)
      return
  450 format (' done ... spots missing hirs ',i5)
  470 format (' pass begins on ',i6,' at ',i6,' gmt, ends at ',i6,
     +       ' gmt',/,6x,'number of lines = ',i4,/,6x,
     +       'satellite ident = ',i2,/,6x,'process  status = ',i1,/,6x,
     +       'direction(node) = ',i1)
      end
